 # SidEngine
 #
 # ICE Team free software group
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
 #  (at your option) any later version.
 #
 #  This program is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
 #  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #  GNU General Public License for more details.
 #
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 #  02111-1307  USA.

use Audio::SID;
use Getopt::Std;

my $NAME = "SidEngin";
my $VERSION = "0.01";

# declare all engine
my @Engine = (
     'Goattrk',
     'Matt Gray engine',
     'Musicdriver 10',
     'Rob Hubbard engine',
     'RockMonitor',
     'Sadotrk'
   );

# declare the table that will contains all the fount sid files divised by engine
my @Table = ([],[],[],[],[],[]);

# declare all reg-espression string
my @Search = (
     # Goattrk
     # search for the code that add arpeggio value to a note
     '\x29\x70'.           # AND  #$70
     '\x4A'.               # LSR  A
     '\x4A'.               # LSR  A
     '\x4A'.               # LSR  A
     '\x4A'.               # LSR  A
     '?'.
     '\x7D..'.             # ADC  $xxxx,X
     '\xA8',               # TAY

     # Matt Gray engine
     # search for the code that read the next pattern value and set the
     # portamento flag
     '\xFE..'.             # INC  $xxxx,X
     '\xD0.'.              # BNE  $xxxx
     '\xC9\xFB'.           # CMP  #$FB
     '\x90\x20'.           # BCC  $xxxx
     '\xC9\xFB'.           # CMP  #$FB
     '\xD0\x18'.           # BNE  $xxxx
     '\xA9\x01'.           # LDA  #$01
     '\x9D..'.             # STA  $xxxx,X
     '\xC8'.               # INY
     '\xFE..'.             # INC  $xxxx,X
     '\xB1',               # LDA  ($xx),Y

     # Musicdriver 10
     # search for code of newtime and add/sub frequency
     '\xDE..'.             # DEC $xxxx,X
     '\xD0\x03'.           # BNE xxxx
     '\xFE..'.             # INC $xxxx,X
     '\xB9..'.             # LDA $xxxx,Y
     '\xA8'.               # TAY
     '\x0A'.               # ASL
     '\x85.'.              # STA $xx
     '\x98'.               # TYA
     '\x30\x0F'.           # BMI xxxx
     '\xBD..'.             # LDA $xxxx,X
     '\x18'.               # CLC
     '\x65.'.              # ADC $xx
     '\x9D..'.             # STA $xxxx,X
     '\x90\x03'.           # BCC xxxx
     '\xFE..'.             # INC $xxxx,X
     '\x60'.               # RTS
     '\xBD..'.             # LDA $xxxx,X
     '\x38'.               # SEC
     '\xE5.'.              # SBC $xx
     '\x9D..'.             # STA $xxxx,X
     '\xB0\x03'.           # BCS xxxx
     '\xDE..'.             # DEC $xxxx,X
     '\x60',               # RTS


     # Rob Hubbard engine
     # search for the code that manipulate the data of the note:
     # first and second byte
     '('.
     '\x2C..'.             # BIT  $xxxx
     '\x70.'.              # BVS  $xxxx
     '\xFE..'.             # INC  $xxxx,X
     '\xAD..'.             # LDA  $xxxx
     '\x10.'.              # BPL  $xxxx
     '\xC8'.               # INY
     '\xB1.'.              # LDA  ($xx),Y
     '\x10.'.              # BPL  $xxxx
     '\x9D..'.             # STA  $xxxx,X
     '\x4C'.               # JMP  $xxxx
     ')|('.

     # search for the code that geth the pitch of the note
     '\xC8'.               # INY
     '\xB1.'.              # LDA  ($xx),Y
     '\x9D..'.             # STA  $xxxx,X
     '\x0A'.               # ASL  A
     '\xA8'.               # TAY
     '.{0,5}'.             # various
     '\xB9..'.             # LDA  $xxxx,Y
     '\x8D..'.             # STA  $xxxx
     '\xB9..'.             # LDA  $xxxx,Y
     '\xAC..'.             # LDY  $xxxx
     '\x99\x01\xD4'.       # STA  $D401,Y
     '\x9D..'.             # STA  $xxxx,X
     '\xAD..'.             # LDA  $xxxx
     '\x99\x00\xD4'.       # STA  $D400,Y
     '\x9D..'.             # STA  $xxxx,X
     '\x4C'.               # JMP  $xxxx
     ')|('.

     # search for the code that manipulate the instrument data structure
     '\xAC..'.             # LDY  $xxxx
     '\xBD..'.             # LDA  $xxxx,X
     '\x8E..'.             # STX  $xxxx
     '\x0A'.               # ASL  A
     '\x0A'.               # ASL  A
     '\x0A'.               # ASL  A
     '\xAA'.               # TAX
     '\xBD..'.             # LDA  $xxxx,X
     '\x8D'.               # STA  $xxxx
     ')',


     # rockmonitor
     # ok, just a simple method
     'ROCKMONITOR',


     # Sadotrk
     # search for code that check for next command (SND,ARP)
     '\xBC..'.             # LDY $xxxx,X
     '\xB1.'.              # LDA ($xx),Y
     '\x30\x2D'.           # BMI xxxx
     '\xC8'.               # INY
     '\xC9\x60'.           # CMP #$60
     '\xB0\x16'.           # BCS xxxx
     '\xC9\x50'.           # CMP #$50
     '\xB0\x03'.           # BCS xxxx
     '\x4C..'.             # JMP $xxxx
     '\xE9\x50'.           # SBC #$50
     '\x84.'.              # STY $xx
     '\xA8'                # TAY
   );

my $mySID = new Audio::SID();
my $directory;                             # directory to scan
my %opts;

sub recurse {
    my ($directory)=$_[0];
    print 'Recurse into directory '.$directory."\n";
    opendir(DIR, $directory);
    my @allFiles = sort {uc($a) cmp uc($b)} readdir(DIR);
    closedir(DIR);

    foreach $file (@allFiles) {
      next if ($file eq '.');
      next if ($file eq '..');
      if (-d $directory.$file) {
        recurse($directory.$file.'/');
        next;
      }

      if ($mySID->read('-filename' => $directory.$file)) {
        my $data = $mySID->get('data');
        for $i (0 .. (scalar(@Search)-1)) {
          if ($data =~ /@Search[$i]/) {
            print $file.':   '.@Engine[$i] ."\n";
            
            push @{ @Table[$i] }, $directory.$file;

          } else {
              ;
            }
        }
      }
      next;
    }
    return 1;
}

sub writeTable {
  for $i (0 .. (scalar(@Engine)-1)) {
    print '****** Engine: '.@Engine[$i]." ******\n";
    for $j (0 .. $#{@Table[$i]}) {
      print ' '.$Table[$i][$j]."\n";
    }
    print "\n";
  }
}

# print usage of the program
sub printUsage {
    print <<EOF;

$NAME version $VERSION

USAGE: SidEngine.pl [-h|?] [-v] [-e] [-b <base dir>]
 base dir: the base directory to scan

-h, -? : prints this help screen.
-e : lists all the engine this version is able to undestand
-v : verbose about the engine used
-b : specify where the base directory is. If ommitted, attempts to
     determine the HVSC base dir from the \$HVSC_BASE environment variable.
     
SidEngine scans the gived directory for sid files and produce a list of known
engine used for those sids.

EOF
}

# print more info to the engines
sub printVerboseEngine {
    print <<EOF;
$NAME version $VERSION

Goattrk:               Goattrk source at http://coverbitops.c64.org
Matt Gray engine:      RE source at SIDin #2 (http://digilander.iol.it/ice00/tsid/sidin/)
Musicdriver 10:        Musicdriver source at http://coverbitops.c64.org
Rob Hubbard engine:    RE source from C-Hacking #5 (http://www.ffd2.com/fridge)
RockMonitor:           simply search for ROCKMONITOR :(((
Sadotrk:               Sadotrk source at http://coverbitops.c64.org

(RE=Reverse Enginnering)

EOF
}

# Print the engine this version is able to find
sub printEngine {
  print $NAME." version ".$VERSION."\n\n";

  print "*** List of the ".scalar(@Engine)." encoded Engines: ***\n";
  for $i (0 .. (scalar(@Engine)-1)) {
    print @Engine[$i]."\n";
  }
}


##############################
#
# Main Program
#
##############################

getopts('eh?vb:', \%opts);

if ($opts{h} or $opts{'?'}) {
    printUsage();
    exit;
}

if ($opts{e}) {
    printEngine();
    exit;
}

if ($opts{v}) {
    printVerboseEngine();
    exit;
}

if ($opts{b}) {
    $directory = $opts{b};
}
else {
    $directory = $ENV{'HVSC_BASE'};
}

#if ($directory) {
#    $directory =- s/\s*$//;
#    $directory =- s/[\/\\]+$//; # Remove trailing slash or backslash, if any.
#}

recurse($directory.'/');

writeTable();
